/* ScummVM - Scumm Interpreter
 * Copyright (C) 2003 The ScummVM project
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Header: /cvsroot/scummvm/scummvm/kyra/kyra.cpp,v 1.2 2004/07/31 09:31:15 fingolfin Exp $
 *
 */

#include "stdafx.h"

#include "base/gameDetector.h"
#include "base/plugins.h"
#include "backends/fs/fs.h"

#include "sound/mixer.h"
#include "common/file.h"
#include "common/config-manager.h"
#include "kyra.h"

#include "resource.h"
#include "VMContext.h"
#include "movies.h"
        
static const GameSettings kyra_settings[] = {
			{ "kyra", "Legend of Kyrandia", 0 },
			{ 0, 0, 0 },
		};

GameList Engine_KYRA_gameList() {
        GameList games;

        games.push_back(kyra_settings[0]);
        return games;
}

// TODO: Improve this :)
DetectedGameList Engine_KYRA_detectGames(const FSList &fslist) {
	DetectedGameList detectedGames;
	//File test_file;

	// Iterate over all files in the given directory
	for (FSList::const_iterator file = fslist.begin(); file != fslist.end(); ++file) {
		const char *name = file->displayName().c_str();
		
		// Detected Kyra Version 
		if (0 == scumm_stricmp("INTRO.SND", name)) {
			// should be Version 1
			detectedGames.push_back(kyra_settings[0]);
		}
	}

	return detectedGames;
}

Engine *Engine_KYRA_create(GameDetector *detector, OSystem *syst) {
	return new Kyra::KyraEngine(detector, syst);
}

REGISTER_PLUGIN("Legend of Kyrandia Engine", Engine_KYRA_gameList, Engine_KYRA_create, Engine_KYRA_detectGames)

namespace Kyra {
KyraEngine::KyraEngine(GameDetector *detector, OSystem *syst)
	: Engine(syst) {

	// Setup mixer
	if (!_mixer->isReady()) {
		warning("Sound initialization failed.");
	}

	_mixer->setVolume(ConfMan.getInt("sfx_volume") * ConfMan.getInt("master_volume") / 255);

	//getGameDataPath();

	// Initialize backend
	syst->initSize(320, 200);
	_plane = new uint8[320*200];
	assert(_plane);
	memset(_plane, 0, sizeof(uint8) * 320 * 200);
	
	_resMgr = new Resourcemanager(this, getGameDataPath());
	assert(_resMgr);
	
	// loads the standard palette for Kyra1	
	setCurrentPalette(_resMgr->loadPalette("PALETTE.COL"));
	
	// loads the 2 cursors
	_mouse = _resMgr->loadImage("MOUSE.CPS");
	_items = _resMgr->loadImage("ITEMS.CPS");
	
	// loads the Font
	_font = _resMgr->loadFont("8FAT.FNT");
	
	// loads out scripts
	_npcScript = _resMgr->loadScript("_NPC.EMC");
	_currentScript = _resMgr->loadScript("_STARTUP.EMC");
}

KyraEngine::~KyraEngine() {
	delete _resMgr;
	delete _currentPal;
	delete _mouse;
	delete _items;
	delete _midiDriver;
	delete _npcScript;
	delete _currentScript;
	delete _font;
}

void KyraEngine::errorString(const char *buf1, char *buf2) {
	strcpy(buf2, "Kyrandia Error: ");
	strcat(buf2, buf1);
}

void KyraEngine::go() {
// This test code is for testing the script interpreter
	// starts the init script
	if (!_currentScript->startScript(kSetupScene)) {
		error("couldn't init '_STARTUP.EMC' script");
	}
	
	if (_currentScript->contScript() != kScriptStopped) {
		if (_currentScript->state() == kScriptError) {
			error("couldn't run script");
		} else {
			warning("init script returned: %d", _currentScript->state());
		}
	}
	
	// code for testing the WSA laoder & renderer
	// the renderer is still buggy it crahses in decode40
	/*Movie* movie = _resMgr->loadMovie("KALLAK.WSA");
	assert(movie);
	
	movie->setRenderedFrames(0, 0xFFFE);
	movie->setDelay(100);*/
	
	while(true) {
		OSystem::Event event;

		//movie->render(_plane, 320, 200);
		updateScreen();
		while (g_system->pollEvent(event)) {
			switch (event.event_code) {
				case OSystem::EVENT_QUIT:
				g_system->quit();
			break;
			
			default:
			break;
    		}
		}
		_system->delayMillis(10);
	}
	
//	delete movie;
}

void KyraEngine::shutdown() {
	_system->quit();
}

#pragma mark ----

void KyraEngine::setCurrentPalette(Palette* pal, bool delNextTime) {
	if (_delPalNextTime)
		delete _currentPal;

	_delPalNextTime = delNextTime;
	
	_currentPal = pal;
	
	if (pal->getData()) {
		_system->setPalette(pal->getData(), 0, 256);
	} else {
		warning("palette contains no data");
	}
}

void KyraEngine::setTempPalette(Palette* pal) {
	if (pal->getData())
		_system->setPalette(pal->getData(), 0, 256);
}

void KyraEngine::upadateCurrentPal(void) {
	if (_currentPal->getData())
		_system->setPalette(_currentPal->getData(), 0, 256);
}

#pragma mark ----

void KyraEngine::drawCursor(void) {
	if (_drawCursor) {
	}
}

void KyraEngine::updateScreen(void) {
	_system->copyRectToScreen(_plane, 320, 0, 0, 320, 240);
	
	// crusor is specially drawed :)
	drawCursor();
	_system->updateScreen();
}

} // End of namespace KYRA

