#include "movies.h"
#include "compression.h"

#include "common/stream.h"

namespace Kyra {
	WSAMovieV1::WSAMovieV1(uint8* data, uint32 size) {
		if (!data) {
			error("resource created without data");
		}

		_buffer = data;

		// I like these Streams .... =)
		Common::MemoryReadStream datastream(data, size);
		datastream.read(&_wsaHeader, sizeof(_wsaHeader));
		
		// check for version
		if (_wsaHeader._type) {
			error("loading a WSA version 2 with the WSA version 1 loader");
		}
		
		uint16 offsetAdd = 0;
		
		// checks now for own palette
		if (_wsaHeader._type % 2) {
			// don't now if this will work right, because a few lines before we use
			// _wsaHeader._type for detect the version of the WSA movie,
			// but this code was from FreeKyra Tools so I think it will work
			
			// if this is a packed palette we have a problem :)
			offsetAdd = 768 /* 0x300 */;
		}
		
		_frameCount = _wsaHeader._numFrames;
		_offsetTable = new uint32[_wsaHeader._numFrames + 2];
		assert(_offsetTable);
		
		// loads the offset table
		for (uint32 tmp = 0; tmp < _wsaHeader._numFrames; ++tmp) {
			_offsetTable[tmp] = datastream.readUint32LE() + offsetAdd;
		}
		
		if (offsetAdd) {
			uint8* palbuffer = new uint8[offsetAdd];
			assert(palbuffer);
			
			// TODO need to load the palette
			
			_ownPalette = new Palette(palbuffer, offsetAdd);
			assert(_ownPalette);
		}
  
		_lastFrameChange = 0;
		_currentFrameNum = 0;
		
		_currentFrame = new uint8[_wsaHeader._width * _wsaHeader._height];
		assert(_currentFrame);
	}
	
	WSAMovieV1::~WSAMovieV1() {
		delete [] _buffer;
		delete [] _offsetTable;
		delete _ownPalette;
	}
	
	const uint8* WSAMovieV1::loadFrame(uint16 frame, uint16* width, uint16* height) {
		if (width) *width = _wsaHeader._width;
		if (height) *height = _wsaHeader._height;
		
		if (frame >= _frameCount) 
			return _currentFrame;
			
		if (frame == _prefetchedFrame) {
			return _currentFrame;
		} else {			
			uint8* frameData = 0;
			static uint8 image40[64000]; // I think this crashes on Plam OS :)
			
			if (frame == _prefetchedFrame + 1) {
				debug("next frame");
				frameData = _buffer + _offsetTable[frame] + (hasPalette() ? 768 : 0);
				debug("decode80");
				Compression::decode80(frameData, image40);
				debug("decode40");
				Compression::decode40(image40, _currentFrame);
				debug("after decode 40");
			} else {
				debug("another frame");
				memset(_currentFrame, 0, _wsaHeader._width * _wsaHeader._height);
				
				for (uint32 i = 0; i <= frame; i++)
				{
					frameData = _buffer + _offsetTable[i] + (hasPalette() ? 768 : 0);
					debug("decode80");
					Compression::decode80(frameData, image40);
					debug("decode40");
					Compression::decode40(image40, _currentFrame);
					debug("after decode 40");
				}
			}
			
			_prefetchedFrame = frame;
			return _currentFrame;
		}
		
		return 0;
	}
	
	void WSAMovieV1::render(uint8* plane, uint16 planepitch, uint16 planeheight) {
		if (_lastFrameChange + _delay < OSystem::instance()->getMillis() || _lastFrameChange == 0) {
			if (_lastFrameChange)
				++_currentFrameNum;
				
			_lastFrameChange = OSystem::instance()->getMillis();
			
			if (_currentFrameNum >= _end_frame || _currentFrameNum >= _frameCount) {
				_currentFrameNum = _start_frame;
			}
			
			debug("beforce loadframe");
			loadFrame(_currentFrameNum, 0, 0);
			debug("loadFrame [OK]");
		}
		
		if (_transparency == -1) {
			// 'fast' blitting
			
			uint8* src = _currentFrame;
			uint8* dst = &plane[_wsaHeader._yPos * planepitch + _wsaHeader._xPos];
			uint32 copysize = planepitch - _wsaHeader._xPos;
			
			if (copysize > _wsaHeader._width)
				copysize = _wsaHeader._width;
			
			for (uint16 y_ = 0; y_ < _wsaHeader._height && _wsaHeader._yPos + y_ < planeheight; ++y_) {
				memcpy(dst, src, copysize * sizeof(uint8));
				dst += planepitch;
				src += _wsaHeader._width;
			}
			
		} else {
			// oh no! we have transparency so we have a very slow copy :/
			uint8* src = _currentFrame;
			uint8* dst = &plane[_wsaHeader._yPos * planepitch + _wsaHeader._xPos];
			
			for (uint16 yadd = 0; yadd < _wsaHeader._height; ++yadd) {
				for (uint16 xadd = 0; xadd < _wsaHeader._width; ++xadd) {
					if (*src == _transparency) {
						++dst;
						++src;
					} else {
						*dst++ = *src++;
					}
				}
				
				dst += planepitch - _wsaHeader._width;
			}
		}
	}
} // end of namespace Kyra

